@@ -7,14 +7,16 @@ its core philosophy and performance characteristics.
 
 from __future__ import annotations
 
+import itertools
 import os
 import sqlite3
 import threading
 import time
 import warnings
+from collections.abc import Iterable
 from enum import Enum
 from pathlib import Path
-from typing import Any, Iterable, Literal, Protocol, cast
+from typing import Any, Literal, Protocol, cast
 
 # Platform-specific imports for file locking
 try:
@@ -39,7 +41,7 @@ from typing_extensions import Self
 
 from ._constants import ConnectionPhase, load_config
 from ._exceptions import DataError, IntegrityError, OperationalError
-from .helpers import _execute_with_retry
+from .helpers import _execute_with_retry, _is_valid_sqlite_db
 
 # Load config once at module level
 _config = load_config()
@@ -132,7 +134,10 @@ class SQLRunner(Protocol):
 class SQLiteRunner:
     """Default synchronous SQLite implementation with thread-local connections."""
 
+    _instance_counter = itertools.count()  # Unique instance ID for debugging
+
     def __init__(self, db_path: str) -> None:
+        self.instance_id = next(self._instance_counter)
         self._db_path = db_path
         self._thread_local = threading.local()
         # Store PID to detect fork
@@ -154,9 +159,9 @@ class SQLiteRunner:
     @property
     def _conn(self) -> sqlite3.Connection:
         """Backward compatibility property for accessing connection."""
-        return self._get_connection()
+        return self.get_connection()
 
-    def _get_connection(self) -> sqlite3.Connection:
+    def get_connection(self) -> sqlite3.Connection:
         """Get or create a thread-local connection.
 
         This ensures each thread has its own SQLite connection, avoiding
@@ -220,15 +225,17 @@ class SQLiteRunner:
 
         return cast("sqlite3.Connection", self._thread_local.conn)
 
-    def _apply_connection_settings(self, conn: sqlite3.Connection) -> None:
+    def _apply_connection_settings(
+        self, conn: sqlite3.Connection, *, config: dict[str, Any] = _config
+    ) -> None:
         """Apply per-connection settings that don't require exclusive locks."""
         # Always set busy timeout for each connection
-        busy_timeout = _config["BROKER_BUSY_TIMEOUT"]
+        busy_timeout = config["BROKER_BUSY_TIMEOUT"]
         conn.execute(f"PRAGMA busy_timeout={busy_timeout}")
 
         # Set WAL autocheckpoint for each connection
         # Default to 1000 pages (≈1MB) if not specified
-        wal_autocheckpoint = _config["BROKER_WAL_AUTOCHECKPOINT"]
+        wal_autocheckpoint = config["BROKER_WAL_AUTOCHECKPOINT"]
         if wal_autocheckpoint < 0:
             warnings.warn(
                 f"Invalid BROKER_WAL_AUTOCHECKPOINT '{wal_autocheckpoint}', "
@@ -268,6 +275,18 @@ class SQLiteRunner:
         # First check SQLite version
         self._check_sqlite_version()
 
+        # Validate that existing database files are actually valid SQLite databases
+        # Only check the SQLite header, not the magic string at this stage
+        # If the file doesn't exist, SQLite will create it during connection
+        db_path = Path(self._db_path)
+        is_new_database = not (db_path.exists() and db_path.stat().st_size > 0)
+
+        if not is_new_database:
+            if not _is_valid_sqlite_db(db_path, verify_magic=False):
+                raise OperationalError(
+                    f"File at {self._db_path} exists but is not a valid SQLite database"
+                )
+
         def do_setup() -> None:
             # Use a separate connection for WAL setup to avoid holding locks
             setup_conn = sqlite3.connect(self._db_path, isolation_level=None)
@@ -275,6 +294,13 @@ class SQLiteRunner:
                 # Set timeout for setup operations
                 setup_conn.execute("PRAGMA busy_timeout=10000")
 
+                # For new databases, set auto_vacuum to INCREMENTAL before any tables are created
+                # This enables automatic space reclamation when rows are deleted
+                if is_new_database:
+                    from ._sql import SET_AUTO_VACUUM_INCREMENTAL
+
+                    setup_conn.execute(SET_AUTO_VACUUM_INCREMENTAL)
+
                 # Check current journal mode
                 cursor = setup_conn.execute("PRAGMA journal_mode")
                 current_mode = cursor.fetchone()[0] if cursor else "delete"
@@ -306,15 +332,17 @@ class SQLiteRunner:
             self._apply_optimization_settings(self._thread_local.conn)
             self._thread_local.optimization_applied = True
 
-    def _apply_optimization_settings(self, conn: sqlite3.Connection) -> None:
+    def _apply_optimization_settings(
+        self, conn: sqlite3.Connection, *, config: dict[str, Any] = _config
+    ) -> None:
         """Apply optimization settings to a connection."""
         # Cache size (default 10MB)
         # Negative values mean KiB (kibibytes), so we multiply by 1024
-        cache_mb = _config["BROKER_CACHE_MB"]
+        cache_mb = config["BROKER_CACHE_MB"]
         conn.execute(f"PRAGMA cache_size=-{cache_mb * 1024}")
 
         # Synchronous mode (default FULL)
-        sync_mode = _config["BROKER_SYNC_MODE"]
+        sync_mode = config["BROKER_SYNC_MODE"]
         # Validate sync mode
         if sync_mode not in ("FULL", "NORMAL", "OFF"):
             warnings.warn(
@@ -334,7 +362,7 @@ class SQLiteRunner:
     ) -> Iterable[tuple[Any, ...]]:
         """Execute SQL and optionally return rows."""
         try:
-            conn = self._get_connection()
+            conn = self.get_connection()
             cursor = conn.execute(sql, params)
             # Only fetch if explicitly requested
             if fetch:
@@ -350,7 +378,7 @@ class SQLiteRunner:
     def begin_immediate(self) -> None:
         """Start an immediate transaction."""
         try:
-            conn = self._get_connection()
+            conn = self.get_connection()
             conn.execute("BEGIN IMMEDIATE")
         except sqlite3.OperationalError as e:
             raise OperationalError(str(e)) from e
@@ -362,7 +390,7 @@ class SQLiteRunner:
     def commit(self) -> None:
         """Commit the current transaction."""
         try:
-            conn = self._get_connection()
+            conn = self.get_connection()
             conn.commit()
         except sqlite3.OperationalError as e:
             raise OperationalError(str(e)) from e
@@ -374,7 +402,7 @@ class SQLiteRunner:
     def rollback(self) -> None:
         """Rollback the current transaction."""
         try:
-            conn = self._get_connection()
+            conn = self.get_connection()
             conn.rollback()
         except sqlite3.OperationalError as e:
             raise OperationalError(str(e)) from e
@@ -423,6 +451,12 @@ class SQLiteRunner:
         if phase in self._completed_phases:
             return
 
+        # Fast-path: check if another process already completed this phase
+        if self._is_phase_already_completed(phase):
+            with self._setup_lock:
+                self._completed_phases.add(phase)
+            return
+
         # Get lock path for this phase
         lock_path = self._get_lock_path(phase)
         if lock_path is None:
@@ -673,3 +707,6 @@ class SQLiteRunner:
         self.cleanup_marker_files()
         self.close()
         return False
+
+
+# ~
